#include "rei-json/Field.h"
#include "rei-json/Array.h"
#include "rei-json/Object.h"
#include <print>
#include <stdexcept>
#include <string>
#include <utility>
namespace rei::json {

	JsonArray::JsonArray() {
		elements.reserve(1000);
	}


	FieldValue& JsonArray::operator[](unsigned index) {
		if (index >= elements.size()) {
			throw std::logic_error(std::string("Trying to access non existing index: ") + std::to_string(index));
		}
		return elements[index];
	}

	JsonArray& JsonArray::push(int value) {
		elements.push_back(value);
		return *this;
	}

	JsonArray& JsonArray::push(FieldValue& value) {
		elements.push_back(value);
		return *this;
	}

	JsonArray& JsonArray::push(FieldValue&& value) {
		elements.push_back(std::move(value));
		return *this;
	}

	JsonArray& JsonArray::push(bool value) {
		elements.push_back(value);
		return *this;
	}

	JsonArray& JsonArray::push(const char * value) {
		elements.push_back(std::string(value));
		return *this;
	}

	JsonArray& JsonArray::push(const std::string& value) {
		elements.push_back(value);
		return *this;
	}

	JsonArray& JsonArray::push(const std::string&& value) {
		elements.push_back(std::move(value));
		return *this;
	}

	JsonArray& JsonArray::pushNull() {
		elements.push_back(std::move(FieldValue{FieldType::Null}));
		return *this;
	}

	JsonArray& JsonArray::push(const JsonObject& object) {
		elements.push_back(object);
		return *this;
	};

	JsonArray& JsonArray::push(const JsonObject&& object) {
		elements.push_back(std::move(object));
		return *this;
	};

	JsonArray& JsonArray::push(const JsonArray& array) {
		elements.push_back(array);
		return *this;
	};

	JsonArray& JsonArray::push(const JsonArray&& array) {
		elements.push_back(std::move(array));
		return *this;
	};

	int JsonArray::getNumber(int index) {
		if (index >= elements.size()) {
			throw std::logic_error(std::string("Trying to access non existing index: ") + std::to_string(index));
		}
		return elements[index].asNumber();;
	}

	bool JsonArray::getBool(int index) {
		if (index >= elements.size()) {
			throw std::logic_error(std::string("Trying to access non existing index: ") + std::to_string(index));
		}
		return elements[index].asBool();;
	}

	std::string JsonArray::getString(int index) {
		if (index >= elements.size()) {
			throw std::logic_error(std::string("Trying to access non existing index: ") + std::to_string(index));
		}
		return elements[index].asString();
	}

	JsonObject& JsonArray::getObject(int index) {
		if (index >= elements.size()) {
			throw std::logic_error(std::string("Trying to access non existing index: ") + std::to_string(index));
		}
		return elements[index].asObject();
	}

	JsonArray& JsonArray::getArray(int index) {
		if (index >= elements.size()) {
			throw std::logic_error(std::string("Trying to access non existing index: ") + std::to_string(index));
		}
		return elements[index].asArray();
	}

	bool JsonArray::hasNull(int index) {
		if (index >= elements.size()) {
			throw std::logic_error(std::string("Trying to access non existing index: ") + std::to_string(index));
		}
		return elements[index].isNull();
	}

	void JsonArray::forEach(std::function<void(FieldValue&, int)> func) {
		for (int i = 0; i < elements.size(); ++i) {
			func(elements[i], i);
		}
	}

	size_t JsonArray::count() const {
		return elements.size();
	}

	void JsonArray::remove(int index) {
		elements.erase(elements.begin() + index);
	}

}
