#include "rei-json/Object.h"
#include "rei-json/json.h"
#include "rei-json/Field.h"
#include "rei-json/Array.h"
#include <print>
#include <stdexcept>
#include <string>
#include <utility>
namespace rei::json {

	JsonObject::JsonObject() {

	};


	FieldValue& JsonObject::operator[](const std::string& key) {
		if (!contains(key)) {
			throw std::logic_error(std::string("Trying to access non existing key: ") + key);
		}
		return elements.at(key);
	}

	bool JsonObject::contains(const std::string& key) const {
		return elements.contains(key);
	}

	JsonObject& JsonObject::set(const std::string& key, FieldValue& value) {
		elements.insert_or_assign(key, value);
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, FieldValue&& value) {
		elements.insert_or_assign(key, std::move(value));
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, int value) {
		elements.insert_or_assign(key, value);
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, bool value) {
		elements.insert_or_assign(key, value);
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, const std::string& value) {
		elements.insert_or_assign(key, value);
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, const std::string&& value) {
		elements.insert_or_assign(key, std::move(value));
		return *this;
	}

	JsonObject& JsonObject::setNull(const std::string& key) {
		elements.insert_or_assign(key, std::move(FieldValue{FieldType::Null}));
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, const char * value) {
		elements.insert_or_assign(key, std::string(value));
		return *this;
	}

	int JsonObject::getNumber(const std::string& key) {
		if (!contains(key)) {
			throw std::logic_error(std::string("Trying to access non existing key: ") + key);
		}
		return elements.at(key).asNumber();
	}

	bool JsonObject::getBool(const std::string& key) {
		if (!contains(key)) {
			throw std::logic_error(std::string("Trying to access non existing key: ") + key);
		}
		return elements.at(key).asBool();
	}

	std::string JsonObject::getString(const std::string& key) {
		if (!contains(key)) {
			throw std::logic_error(std::string("Trying to access non existing key: ") + key);
		}
		return elements.at(key).asString();
	}

	JsonObject& JsonObject::getObject(const std::string& key) {
		if (!contains(key)) {
			throw std::logic_error(std::string("Trying to access non existing key: ") + key);
		}
		return elements.at(key).asObject();
	}

	JsonArray& JsonObject::getArray(const std::string& key) {
		if (!contains(key)) {
			throw std::logic_error(std::string("Trying to access non existing key: ") + key);
		}
		return elements.at(key).asArray();
	}

	bool JsonObject::hasNull(const std::string& key) {
		if (!contains(key)) {
			throw std::logic_error(std::string("Trying to access non existing key: ") + key);
		}
		return elements.at(key).isNull();
	}

	JsonObject& JsonObject::set(const std::string& key, const JsonArray& array) {
		elements.insert_or_assign(key, array);
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, JsonArray&& array) {
		elements.insert_or_assign(key, std::move(array));
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, const JsonObject& object) {
		elements.insert_or_assign(key, object);
		return *this;
	}

	JsonObject& JsonObject::set(const std::string& key, JsonObject&& object) {
		elements.insert_or_assign(key, std::move(object));
		return *this;
	}

	void JsonObject::forEach(std::function<void(FieldValue&, const std::string& key)> func) {
		for (auto& element : elements) {
			func(element.second, element.first);
		}
	}

	size_t JsonObject::count() const {
		return elements.size();
	}

	void JsonObject::remove(const std::string& key) {
		elements.erase(key);
	}
}
