#include "rei-json/Array.h"
#include "rei-json/Field.h"
#include "rei-json/Object.h"
#include "rei-json/json.h"
#include <cctype>
#include <print>
#include <stdexcept>
#include <string>
#include <variant>
namespace  rei::json {

	void arrayToString(JsonArray& array, std::string& str, int currentIndentation, int indentIncrement);

	inline void objectToString(JsonObject& object, std::string& str, int currentIndentation, int indentIncrement) {
		int elementCount = object.count();
		//std::string str = "{";
		str += "{";
		currentIndentation += indentIncrement;
		std::string padding = "\n" + std::string(currentIndentation, ' ');
		object.forEach([&](FieldValue& element, const std::string& key) {
			str += padding + "\"" + key + "\": ";
			if (element.isNumber()) {
				str += std::to_string(element.asNumber());
			} else if (element.isBool()) {
				str += element.asBool() ? "true" : "false";
			} else if (element.isNull()) {
				str += "null";
			} else if (element.isString()) {
				str += "\"" + element.asString() + "\"";
			} else if (element.isObject()) {
				objectToString(element.asObject(), str, currentIndentation, indentIncrement);
			} else if (element.isArray()) {
				arrayToString(element.asArray(), str, currentIndentation, indentIncrement);
			} 
			elementCount--;
			if (elementCount > 0) {
				str += ",";
			}
		});
		currentIndentation -= indentIncrement;
		if (object.count() > 0) {
			str += "\n" + std::string(currentIndentation, ' ');
		}
		str += "}";
		//return str;
	}

	inline void arrayToString(JsonArray& array, std::string& str, int currentIndentation, int indentIncrement) {
		int elementCount = array.count();
		//std::string str = "[";
		str += "[";
		currentIndentation += indentIncrement;
		std::string padding = "\n" + std::string(currentIndentation, ' ');
		array.forEach([&](FieldValue& element, int index) {
			str += padding;
			if (element.isNumber()) {
				str += std::to_string(element.asNumber());
			} else if (element.isBool()) {
				str += element.asBool() ? "true" : "false";
			} else if (element.isNull()) {
				str += "null";
			} else if (element.isString()) {
				str += "\"" + element.asString() + "\"";
			} else if (element.isObject()) {
				objectToString(element.asObject(), str, currentIndentation, indentIncrement);
			} else if (element.isArray()) {
				arrayToString(element.asArray(), str, currentIndentation, indentIncrement);
			}   
			--elementCount;
			if (elementCount > 0) {
				str += ",";
			}
		});
		currentIndentation -= indentIncrement;
		if (array.count() > 0) {
			str += "\n" + std::string(currentIndentation, ' ');
		}
		str += "]";
		//return str;
	}

	std::string toString(std::variant<JsonObject, JsonArray>& element) {
		if (std::holds_alternative<JsonObject>(element)) {
			return toString(std::get<JsonObject>(element));
		} else if (std::holds_alternative<JsonArray>(element)) {
			return toString(std::get<JsonArray>(element));

		}
		throw std::runtime_error("Bad json format");
	}

	std::string toString(JsonObject& object) {
		std::string str;
		objectToString(object, str, 0, 4);
		return str;
	}

	std::string toString(JsonArray& array) {
		std::string str;
		arrayToString(array, str, 0, 4);
		return str;
	}
}
